@@ -29,7 +29,7 @@
 #' @param page Integer. Which page of records to start downloading from.
 #' @param count_only Logical. Only return the number of records available.
 #' @param quiet Logical. Suppress the progress indicator for multipage
-#'   downloads.
+#'   downloads. Defaults to value of option `finbif_hide_progress`.
 #' @param cache Logical. Use cached data.
 #' @param dwc Logical. Use Darwin Core (or Darwin Core style) variable names.
 #' @param seed Integer. Set a seed for randomly sampling records.
@@ -37,6 +37,12 @@
 #'   returned as an attribute?
 #' @param exclude_na Logical. Should records where all selected variables have
 #'   non-NA values only be returned.
+#' @param locale Character. One of the supported two-letter ISO 639-1 language
+#'   codes. Current supported languages are English, Finnish, Swedish, Russian,
+#'   and Sámi (Northern). For data where more than one language is available
+#'   the language denoted by `locale` will be preferred while falling back to
+#'   the other languages in the order indicated above.
+#' @param include_facts Logical. Should all "fact" variables be included?
 #' @return A `finbif_api` or `finbif_api_list` object.
 #' @examples \dontrun{
 #'
@@ -48,8 +54,9 @@
 
 finbif_records <- function(
   filter, select, order_by, aggregate, sample = FALSE, n = 10, page = 1,
-  count_only = FALSE, quiet = FALSE, cache = getOption("finbif_use_cache"),
-  dwc = FALSE, seed, df = FALSE, exclude_na = FALSE
+  count_only = FALSE, quiet = getOption("finbif_hide_progress"),
+  cache = getOption("finbif_use_cache"), dwc = FALSE, seed, df = FALSE,
+  exclude_na = FALSE, locale = getOption("finbif_locale"), include_facts = FALSE
 ) {
 
   max_size <- getOption("finbif_max_page_size")
@@ -81,7 +88,7 @@ finbif_records <- function(
 
     # select ===================================================================
 
-    select <- infer_selection(aggregate, select, var_type)
+    select <- infer_selection(aggregate, select, include_facts, var_type)
 
     select_param <- switch(aggregate[[1L]], none = "selected", "aggregateBy")
 
@@ -122,12 +129,12 @@ finbif_records <- function(
   ans <- request(
     filter, select[["query"]], sample, n, page, count_only, quiet, cache, query,
     max_size, select[["user"]], select[["record_id_selected"]], dwc, aggregate,
-    df, seed, exclude_na
+    df, seed, exclude_na, locale, include_facts
   )
 
   if (df && !count_only) {
     ind <- length(ans)
-    attr(ans[[ind]], "df") <- as.data.frame(ans[[ind]])
+    attr(ans[[ind]], "df") <- as.data.frame(ans[[ind]], locale = locale)
   }
 
   ans
@@ -164,7 +171,7 @@ infer_aggregation <- function(aggregate) {
 
 # selection --------------------------------------------------------------------
 
-infer_selection <- function(aggregate, select, var_type) {
+infer_selection <- function(aggregate, select, include_facts, var_type) {
 
   date_time_vars <- var_names[var_names[["date"]], ]
 
@@ -184,21 +191,6 @@ infer_selection <- function(aggregate, select, var_type) {
     "aggregate"
   )
 
-  abundance_vars <- c(
-    "unit.interpretations.individualCount", "unit.abundanceString"
-  )
-
-  coordinates_uncertainty_vars <- c(
-    "gathering.interpretations.coordinateAccuracy", "document.sourceId"
-  )
-
-  citation_vars <- c("document.documentId", "document.sourceId")
-
-  scientific_name_vars <- c(
-    "unit.linkings.taxon.scientificName", "unit.taxonVerbatim",
-    "document.sourceId"
-  )
-
   if (missing(select)) {
 
     select <- row.names(default_vars)
@@ -207,11 +199,14 @@ infer_selection <- function(aggregate, select, var_type) {
 
     if (identical(aggregate, "none")) {
       # Missing 'select' implies default selection which implies date-time,
-      # abundance and coord uncertainty and scientific name calc needed
+      # abundance, coord uncertainty and scientific name calc needed
       select <- unique(
         c(
-          select, row.names(date_time_vars), abundance_vars,
-          coordinates_uncertainty_vars, scientific_name_vars
+          select, row.names(date_time_vars),
+          "unit.interpretations.individualCount", "unit.abundanceString",
+          "gathering.interpretations.coordinateAccuracy",
+          "unit.linkings.taxon.scientificName", "unit.taxonVerbatim",
+          "document.sourceId"
         )
       )
       record_id_selected <- TRUE
@@ -232,7 +227,9 @@ infer_selection <- function(aggregate, select, var_type) {
     record_id_selected <- var_names["unit.unitId", var_type] %in% select
 
     if (!record_id_selected && identical(aggregate, "none")) {
+
       select <- c(var_names["unit.unitId", var_type], select)
+
     }
 
     vars <- c(
@@ -240,63 +237,18 @@ infer_selection <- function(aggregate, select, var_type) {
       "duration", "samplingEffort"
     )
 
-    date_time <- any(vars %in% select)
+    if (any(vars %in% select)) {
 
-    if (date_time) {
       select <- unique(c(select, date_time_vars[[var_type]]))
-    }
-
-    vars <- c(
-      "abundance", "individualCount", "occurrence_status", "occurrenceStatus"
-    )
-
-    abundance <- any(vars %in% select)
-
-    if (abundance) {
-
-      select <- unique(c(select, var_names[abundance_vars, var_type]))
-
-    }
-
-    vars <- c("coordinates_uncertainty", "coordinateUncertaintyInMeters")
-
-    coordinates_uncertainty <- any(vars %in% select)
-
-    if (coordinates_uncertainty) {
-
-      select <- unique(
-        c(select, var_names[coordinates_uncertainty_vars, var_type])
-      )
-
-    }
-
-    vars <- c("citation", "bibliographicCitation")
-
-    citation <- any(vars %in% select)
-
-    if (citation) {
-
-      select <- unique(
-        c(select, var_names[citation_vars, var_type])
-      )
-
-    }
-
-    vars <- c("scientific_name", "scientificName")
-
-    scientific_name <- any(vars %in% select)
-
-    if (scientific_name) {
-
-      select <- unique(
-        c(select, var_names[scientific_name_vars, var_type])
-      )
 
     }
 
+    select <- infer_computed_vars(select, var_type)
 
     select_vars <- var_names[var_names[[select_type]], var_type, drop = FALSE]
+
     class(select_vars[[var_type]]) <- class(var_names[[var_type]])
+
     select <- translate(
       select, "select_vars", list(select_vars = select_vars)
     )
@@ -323,18 +275,100 @@ infer_selection <- function(aggregate, select, var_type) {
 
   }
 
+  if (include_facts) {
+
+    select <- c(
+      select, "unit.facts.fact", "unit.facts.value", "gathering.facts.fact",
+      "gathering.facts.value", "document.facts.fact", "document.facts.value"
+    )
+
+  }
+
   # Can't query the server for vars that are computed after download
-  select <- select[!grepl("^computed_var", select)]
+  select <- unique(select[!grepl("^computed_var", select)])
 
   list(query = select, user = select_, record_id_selected = record_id_selected)
 
 }
 
+infer_computed_vars <- function(select, var_type) {
+
+  abundance_vars <- c(
+    "abundance", "individualCount", "occurrence_status", "occurrenceStatus"
+  )
+
+  if (any(abundance_vars %in% select)) {
+
+    abundance_vars <- c(
+      "unit.interpretations.individualCount", "unit.abundanceString"
+    )
+
+    select <- unique(c(select, var_names[abundance_vars, var_type]))
+
+  }
+
+  coordinates_uncertainty_vars <- c(
+    "coordinates_uncertainty", "coordinateUncertaintyInMeters"
+  )
+
+  if (any(coordinates_uncertainty_vars %in% select)) {
+
+    coordinates_uncertainty_vars <- c(
+      "gathering.interpretations.coordinateAccuracy", "document.sourceId"
+    )
+
+    select <- unique(
+      c(select, var_names[coordinates_uncertainty_vars, var_type])
+    )
+
+  }
+
+  citation_vars <- c("citation", "bibliographicCitation")
+
+  if (any(citation_vars %in% select)) {
+
+    citation_vars <- c("document.documentId", "document.sourceId")
+
+    select <- unique(c(select, var_names[citation_vars, var_type]))
+
+  }
+
+  scientific_name_vars <- c("scientific_name", "scientificName")
+
+  if (any(scientific_name_vars %in% select)) {
+
+    scientific_name_vars <- c(
+      "unit.linkings.taxon.scientificName", "unit.taxonVerbatim",
+      "document.sourceId"
+    )
+
+    select <- unique(c(select, var_names[scientific_name_vars, var_type]))
+
+  }
+
+  red_list_vars <- c("red_list_status", "redListStatus")
+
+  if (any(red_list_vars %in% select)) {
+
+    red_list_vars <- c(
+      "unit.linkings.taxon.latestRedListStatusFinland.status",
+      "unit.linkings.taxon.latestRedListStatusFinland.year"
+    )
+
+    select <- unique(c(select, var_names[red_list_vars, var_type]))
+
+  }
+
+  select
+
+}
+
 # request ----------------------------------------------------------------------
 
 request <- function(
   filter, select, sample, n, page, count_only, quiet, cache, query, max_size,
-  select_, record_id_selected, dwc, aggregate, df, seed, exclude_na
+  select_, record_id_selected, dwc, aggregate, df, seed, exclude_na, locale,
+  include_facts
 ) {
 
   path <- getOption("finbif_warehouse_query")
@@ -380,7 +414,8 @@ request <- function(
     if (sample && sample_after_request) {
       all_records <- finbif_records(
         filter, select_, sample = FALSE, n = n_tot, quiet = quiet,
-        cache = cache, dwc = dwc, df = df, exclude_na = exclude_na
+        cache = cache, dwc = dwc, df = df, exclude_na = exclude_na,
+        locale = locale, include_facts = include_facts
       )
       return(record_sample(all_records, n, cache))
     }
@@ -394,7 +429,8 @@ request <- function(
       if (missing(seed)) seed <- 1L
 
       resp <- handle_duplicates(
-        resp, filter, select_, max_size, cache, n, seed, dwc, df, exclude_na
+        resp, filter, select_, max_size, cache, n, seed, dwc, df, exclude_na,
+        locale, include_facts
       )
 
     }
@@ -642,7 +678,8 @@ record_sample <- function(x, n, cache) {
 # handle duplicates ------------------------------------------------------------
 
 handle_duplicates <- function(
-  x, filter, select, max_size, cache, n, seed, dwc, df, exclude_na
+  x, filter, select, max_size, cache, n, seed, dwc, df, exclude_na, locale,
+  include_facts
 ) {
 
   ids <- lapply(
@@ -665,13 +702,15 @@ handle_duplicates <- function(
 
     new_records <- finbif_records(
       filter, select, sample = TRUE, n = max_size, cache = cache, dwc = dwc,
-      seed = seed, df = df, exclude_na = exclude_na
+      seed = seed, df = df, exclude_na = exclude_na, locale = locale,
+      include_facts = include_facts
     )
 
     x[[length(x) + 1L]] <- new_records[[1L]]
 
     x <- handle_duplicates(
-      x, filter, select, max_size, cache, n, seed + 1L, dwc, df, exclude_na
+      x, filter, select, max_size, cache, n, seed + 1L, dwc, df, exclude_na,
+      locale, include_facts
     )
 
   }
